package csc8711.nosql;

import java.util.ArrayList;

import nl.renarj.jasdb.api.DBSession;
import nl.renarj.jasdb.api.SimpleEntity;
import nl.renarj.jasdb.api.model.EntityBag;
import nl.renarj.jasdb.api.query.QueryBuilder;
import nl.renarj.jasdb.api.query.QueryExecutor;
import nl.renarj.jasdb.api.query.QueryResult;
import nl.renarj.jasdb.rest.client.RestDBSession;
import nl.renarj.jasdb.core.exceptions.*;
import nl.renarj.core.exceptions.ConfigurationException;

public class JasDbConnection {

    private DBSession session;
    private EntityBag userBag;
    private EntityBag inventoryBag;
    private EntityBag categoryBag;

    public JasDbConnection() throws JasDBStorageException, ConfigurationException {
        session = new RestDBSession("default", "localhost", 7050);  
        userBag = session.createOrGetBag("user");
        inventoryBag = session.createOrGetBag("inventory");
        categoryBag = session.createOrGetBag("category");
        
}
   
   // remove bag
   public void removeBag(String name) throws JasDBStorageException {  	
			session.removeBag(name);
   }
   
   // find user
   public boolean findUser (String userid, String password) throws JasDbException, JasDBStorageException { 
	   QueryExecutor executor = userBag.find(QueryBuilder.createBuilder().field("id")
				   .value(userid).field("password").value(password));
		   QueryResult result = executor.execute();
		   if(result.size()==0) {
			   throw new JasDbException("No User Found");
		   }
		   if(result.size()>1) {
			   throw new JasDbException("Multiple users");
		   } 
		   
		   return true;
   }
   // find user
   public void addCategory (String name, String description) throws JasDbException, JasDBStorageException {

		   QueryExecutor executor = categoryBag.find(QueryBuilder.createBuilder().field("category")
				   .value(name));
		   QueryResult result = executor.execute();
		   if(result.size()>1) {
			   throw new JasDbException("Category already exists");
		   }
		   if(result.size()==0) {
			   SimpleEntity entity = new SimpleEntity();
			   entity.addProperty("category", name);
			   entity.addProperty("description", description);
			   categoryBag.addEntity(entity);
		   }
		   
   }
   public void removeCategory(String id) throws JasDbException, JasDBStorageException {
	   SimpleEntity entity = categoryBag.getEntity(id);
	   if(entity == null) {
		   throw new JasDbException("Category item doesn't exists");
	   }
	   categoryBag.removeEntity(entity);
   }
   
   public void updateCategory(String id, String category, String description) throws JasDbException, JasDBStorageException {
	   SimpleEntity entity = categoryBag.getEntity(id);
	   if(entity == null) {
		   throw new JasDbException("Category item doesn't exists");   
	   }
	   entity.setProperty("description", description);
	   entity.setProperty("category", category);;
	   categoryBag.updateEntity(entity); 
   }
   
   public String getCategoryName(String id) throws JasDbException, JasDBStorageException {
	   SimpleEntity entity = categoryBag.getEntity(id);
	   if(entity == null) {
		  return null;
	   }
	   return entity.getValue("category").toString();
   }
   
   public  boolean inventoryByCategory(String id) throws JasDbException, JasDBStorageException {
	   QueryExecutor executor = inventoryBag.find(QueryBuilder.createBuilder().field("category").value(id));
	   QueryResult result = executor.execute();
	   if(result.size()>0) {
		   return false;
	   }
	   return true;
	   
   }
   public String getCategoryID(String category) throws JasDbException, JasDBStorageException {
	   QueryExecutor executor = categoryBag.find(QueryBuilder.createBuilder().field("category").value(category));
	   QueryResult result = executor.execute();
	   String id = null;
	   if(result.size()==1) {
		   for(SimpleEntity entity: result ) {
			   id = entity.getValue("__ID").toString();
	   		}
		   return id;
	   } 
	   return null;
	  
   }
   
   public QueryResult viewCategory() throws JasDbException, JasDBStorageException {
	   return categoryBag.getEntities();
   }
   public void addInventory (String name, String description, String category, String price, String count) throws JasDbException, JasDBStorageException {
	   String categoryID = getCategoryID(category);
	   QueryExecutor executor = inventoryBag.find(QueryBuilder.createBuilder().field("name").value(name).field("category").value(categoryID));
	   QueryResult result = executor.execute();
	   if(result.size()>0) {
		   throw new JasDbException("Inventory item already exists");
		   
	   }

	   SimpleEntity entity = new SimpleEntity();
	   	entity.addProperty("name", name);
		entity.addProperty("description", description);
		entity.addProperty("category", categoryID);
		entity.addProperty("price",price);
		entity.addProperty("count",count);
	    inventoryBag.addEntity(entity);

	   
   }
   
   public void updateInventory (String id, String name, String description, String categoryID, String price, String count) throws JasDbException, JasDBStorageException {
	   SimpleEntity entity = inventoryBag.getEntity(id);
	   if(entity == null) {
		   throw new JasDbException("Inventory item doesn't exists");   
	   }
			entity.setProperty("name", name);
			entity.setProperty("description", description);
			entity.setProperty("category", categoryID);
			entity.setProperty("price",price);
			entity.setProperty("count",count);
		    inventoryBag.updateEntity(entity); 
   
   }
   
   public void removeInventory (String id) throws JasDbException, JasDBStorageException {
	   SimpleEntity entity = inventoryBag.getEntity(id);
	   if(entity == null) {
		   throw new JasDbException("Inventory item doesn't exists");
	   }
	   inventoryBag.removeEntity(entity);
	   
   }
   public void addUser (String userid, String password, String name, String role) throws JasDbException, JasDBStorageException {
	   QueryExecutor executor = userBag.find(QueryBuilder.createBuilder().field("userid").value(userid));
	   QueryResult result = executor.execute();
	   if(result.size()>0) {
		   throw new JasDbException("Userid already exists");
		   
	   }

	   SimpleEntity entity = new SimpleEntity();
	   	entity.addProperty("id", userid);
		entity.addProperty("name", name);
		entity.addProperty("password", password);
		entity.addProperty("role",role);
	    userBag.addEntity(entity);
	   
   }
   public void removeUser (String id) throws JasDbException, JasDBStorageException {
	   SimpleEntity entity = userBag.getEntity(id);
	   if(entity == null) {
		   throw new JasDbException("Inventory item doesn't exists");
	   }
	   userBag.removeEntity(entity);
	   
   }
   
   public void updateUser (String id, String userid, String name, String password, String role) throws JasDbException, JasDBStorageException {
	   SimpleEntity entity = userBag.getEntity(id);
	   if(entity == null) {
		   throw new JasDbException("User doesn't exists");   
	   }
	   		entity.setProperty("userid", userid);
	   		entity.setProperty("name", name);
			entity.setProperty("password", password);		
			entity.setProperty("role",role);
		    userBag.updateEntity(entity); 
   
   }
   public ArrayList<CategoryItem> getCategory () throws JasDbException, JasDBStorageException {
	   ArrayList<CategoryItem> myArr = new ArrayList<CategoryItem>();
	   QueryResult result = categoryBag.getEntities();
	   
	   String id, description,category=null;
	   for(SimpleEntity entity : result) {
		   id = entity.getValue("__ID").toString();
		   if(entity.hasProperty("category"))
			   category = entity.getValue("category").toString();
		   else
			   category = "";
		   if( entity.hasProperty("description")) 
			   description = entity.getValue("description").toString();
		   else
			   description = "";
		  
		   CategoryItem item = new CategoryItem(id,category,description);  
	   	   myArr.add(item);
	    

		}
	   return myArr;
   }
   public ArrayList<InventoryItem> getInventory () throws JasDbException, JasDBStorageException {
	   ArrayList<InventoryItem> myArr = new ArrayList<InventoryItem>();
	   QueryResult result = inventoryBag.getEntities();
	   
	   String id, name,description,category,price,count=null;
	   for(SimpleEntity entity : result) {
		   id = entity.getValue("__ID").toString();
		   if(entity.hasProperty("name"))
			   name = entity.getValue("name").toString();
		   else
			   name = "";
		   if( entity.hasProperty("description")) 
			   description = entity.getValue("description").toString();
		   else
			   description = "";
		   
		   if( entity.hasProperty("category")) {
			   String categoryID = entity.getValue("category").toString();
			   category = getCategoryName(categoryID);
		   } else
			   category = "";
		   
		   if (entity.hasProperty("price"))
			   price = entity.getValue("price").toString();
		   else 
			   price = "";
		   if (entity.hasProperty("count"))
			   count = entity.getValue("count").toString();
		   else 
			   count = ""; 
		  
		   InventoryItem item = new InventoryItem(id,name,description,category,price,count);  
	   	   myArr.add(item);
	    

		}
	   return myArr;
   }
   public ArrayList<User> getUsers() throws JasDbException, JasDBStorageException {
	   ArrayList<User> myArr = new ArrayList<User>();
	   QueryResult result = userBag.getEntities();   
	   String id, userid,name,password,role=null;
	   for(SimpleEntity entity : result) {
		   id = entity.getValue("__ID").toString();
		   if(entity.hasProperty("id"))
			   userid = entity.getValue("id").toString();
		   else
			   userid = "";
		   if( entity.hasProperty("name")) 
			   name = entity.getValue("name").toString();
		   else
			   name = "";
		   if( entity.hasProperty("password")) 
			   password = entity.getValue("password").toString();
		   else
			   password = "";
		   if( entity.hasProperty("role")) 
			   role = entity.getValue("role").toString();
		   else
			  role = "";
		   User item = new User(id,userid,name,password,role);  
	   	   myArr.add(item);
	    

		}
	   return myArr;
   }
   public boolean isEmpty(String name) throws JasDbException, JasDBStorageException {
	   if(name.equals("user")) {
		   if(userBag.getSize()==0) return true;
		   return false;
	   } else if (name.equals("inventory")) {
		   if(inventoryBag.getSize()==0) return true;
		   return false;
	   } else if (name.equals("inventory")) {
		   if(categoryBag.getSize()==0) return true;
		   return false;
	   }
	   return false;
	   
   }
}
